package com.bwie.auth.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.nacos.common.utils.RandomUtils;
import com.bwie.auth.feign.AuthFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.request.UserReq;
import com.bwie.common.domain.response.JwtResp;
import com.bwie.common.domain.response.UserResp;
import com.bwie.common.exception.ProductException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.regex.Pattern;

/**
 * @Author：weiran
 * @Package：com.bwie.auth.service.impl
 * @Project：week3
 * @name：AuthServiceImpl
 * @Date：2024/8/6 9:52
 */
@Service
public class AuthServiceImpl implements AuthService {
    @Autowired
    private AuthFeign authFeign;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;


    /**
     * 发送验证码
     * @param
     * @return
     */
    @Override
    public String sendcode(String userPhone) {
        //判断手机号是否为空
        if (StringUtils.isBlank(userPhone)){
            throw new ProductException("手机号不能为空");
        }
        //判断手机号格式是否正确
        boolean matches = Pattern.matches("^(?:(?:\\+|00)86)?1\\d{10}$", userPhone);
        if (!matches){
            throw new ProductException("手机号格式不正确");
        }
        //判断手机号是否存在
        Result<UserResp> phone = authFeign.findByuserPhone(userPhone);
        UserResp resp = phone.getData();
        if (resp==null){
            throw new ProductException("手机号不存在");
        }

        //创建验证码
        String code = RandomUtil.randomNumbers(4);
        //将验证码存入redis
        redisTemplate.opsForValue().set(userPhone, code);

        return code;

    }

    /**
     * 登录
     * @param userReq
     * @return
     */
    @Override
    public JwtResp login(UserReq userReq) {
        //判断手机号或验证码是否为空
        if (StringUtils.isAnyBlank(userReq.getUserPhone(),userReq.getCode())){
            throw new ProductException("手机号或验证码不能为空");
        }
        //判断手机号是否存在
        Result<UserResp> phone = authFeign.findByuserPhone(userReq.getUserPhone());
        UserResp resp = phone.getData();
        if (resp==null){
            throw new ProductException("手机号不存在");
        }
        //判读验证码是否过期
        Boolean aBoolean = redisTemplate.hasKey(userReq.getUserPhone());
        if (!aBoolean){
            throw new ProductException("验证码已过期");
        }
        //判断验证码是否正确
        String code = redisTemplate.opsForValue().get(userReq.getUserPhone());
        if (!code.equals(userReq.getCode())){
            throw new ProductException("验证码不正确");
        }

        //登录
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY, resp.getUserId());
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+resp.getUserId(), JSON.toJSONString(resp));
        JwtResp jwtResp = new JwtResp();
        jwtResp.setToken(token);
        jwtResp.setExpirtime("30MIN");
        return jwtResp;
    }

    @Override
    public UserResp getInfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String code = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSON.parseObject(code, UserResp.class);

    }
}
